-----------------------------------------------------------------------------------------------
-- Client Lua Script for DistanceToTarget
-- Copyright (c) NCsoft. All rights reserved
-----------------------------------------------------------------------------------------------
 
require "Window"
 
-----------------------------------------------------------------------------------------------
-- DistanceToTarget Module Definition
-----------------------------------------------------------------------------------------------
local DistanceToTarget = {} 
 
-----------------------------------------------------------------------------------------------
-- Constants
-----------------------------------------------------------------------------------------------
-- e.g. local kiExampleVariableMax = 999
 
-----------------------------------------------------------------------------------------------
-- Initialization
-----------------------------------------------------------------------------------------------
function DistanceToTarget:new(o)
    o = o or {}
    setmetatable(o, self)
    self.__index = self 

    -- initialize variables here

    return o
end

function DistanceToTarget:Init()
    Apollo.RegisterAddon(self, false)
end
 

-----------------------------------------------------------------------------------------------
-- DistanceToTarget OnLoad
-----------------------------------------------------------------------------------------------
function DistanceToTarget:OnLoad()
    self.config = Apollo.GetPackage("Gemini:DB-1.0").tPackage:New(self)
	self.xmlDoc = XmlDoc.CreateFromFile("DistanceToTarget.xml")
	self.xmlDoc:RegisterCallback("OnDocLoaded", self)
end

-----------------------------------------------------------------------------------------------
-- DistanceToTarget OnDocLoaded
-----------------------------------------------------------------------------------------------
function DistanceToTarget:OnDocLoaded()

	if self.xmlDoc ~= nil and self.xmlDoc:IsLoaded() then
	    self.wndMain = Apollo.LoadForm(self.xmlDoc, "MainWindow", nil, self)
		if self.wndMain == nil then
			Apollo.AddAddonErrorText(self, "Could not load the main window for some reason.")
			return
		end
		self:initLocation()
	    self.wndMain:Show(true)
		self.timer = ApolloTimer.Create(1.000, true, "OnTimer", self)
	end
end

function DistanceToTarget:initLocation()
	if self.config.char.Position ~= nil then
		self.wndMain:SetAnchorOffsets(self.config.char.Position.left, self.config.char.Position.top, self.config.char.Position.right, self.config.char.Position.bottom)
	else 
		self.config.char.Position = {}
		self.config.char.Position.left, self.config.char.Position.top, self.config.char.Position.right, self.config.char.Position.bottom = self.wndMain:GetAnchorOffsets()
	end
end

-----------------------------------------------------------------------------------------------
-- DistanceToTarget Functions
-----------------------------------------------------------------------------------------------
-- on timer
function DistanceToTarget:OnTimer()
	if GameLib:GetPlayerUnit() == nil then
		self.wndMain:Show(false)
		return
	end
	local target = GameLib:GetPlayerUnit():GetTarget()
	if target == nil then
		self.wndMain:Show(false)
		return
	else
		self.wndMain:Show(true)
		local tPos = target:GetPosition()
		local actualPosition = GameLib.GetPlayerUnit():GetPosition()
		self:UpdateContent(self:GetDistance(tPos, actualPosition))
		return
	end
end

function DistanceToTarget:UpdateContent(distance)
	self.wndMain:SetText(distance .. "m");
end

function DistanceToTarget:GetDistance(pos1, pos2)
	local dZ = pos2.z - pos1.z
	local dY = pos2.y - pos1.y
	local dX = pos2.x - pos1.x
	
	local dist = math.sqrt((dZ*dZ)+(dY*dY)+(dX*dX))
	
	return self:round(dist, 0)
end

function DistanceToTarget:round(num, idp)
  local mult = 10^(idp or 0)
  return math.floor(num * mult + 0.5) / mult
end

---------------------------------------------------------------------------------------------------
-- MainWindow Functions
---------------------------------------------------------------------------------------------------

function DistanceToTarget:onWindowChange( wndHandler, wndControl, nOldLeft, nOldTop, nOldRight, nOldBottom )
--SendVarToRover("Character", GameLib:GetPlayerUnit());
--SendVarToRover("GameLib", GameLib);
	self.config.char.Position.left, self.config.char.Position.top, self.config.char.Position.right, self.config.char.Position.bottom =  self.wndMain:GetAnchorOffsets()
end

-----------------------------------------------------------------------------------------------
-- DistanceToTarget Instance
-----------------------------------------------------------------------------------------------
local DistanceToTargetInst = DistanceToTarget:new()
DistanceToTargetInst:Init()
